package logic

import (
	"context"
	"git.noahlan.cn/northlan/ntools-go/uuid"
	"github.com/pkg/errors"
	"live-service/app/user_center/model"
	"live-service/app/user_center/rpc/internal/logic/platform_user"
	"time"

	"live-service/app/user_center/rpc/internal/svc"
	"live-service/app/user_center/rpc/pb"

	"github.com/zeromicro/go-zero/core/logx"
)

type RetrievePlatformUserLogic struct {
	ctx                   context.Context
	svcCtx                *svc.ServiceContext
	platformUserRetriever *platform_user.UserRetriever

	logx.Logger
}

func NewRetrievePlatformUserLogic(ctx context.Context, svcCtx *svc.ServiceContext) *RetrievePlatformUserLogic {
	return &RetrievePlatformUserLogic{
		ctx:                   ctx,
		svcCtx:                svcCtx,
		platformUserRetriever: platform_user.NewUserRetriever(),
		Logger:                logx.WithContext(ctx),
	}
}

// RetrievePlatformUser 查询或创建用户
func (l *RetrievePlatformUserLogic) RetrievePlatformUser(in *pb.PlatformUserReq) (*pb.PlatformUserResp, error) {
	var err error
	dbPlatformUser, err := l.svcCtx.UserPlatformModel.FindOneByPlatformAndPUid(l.ctx, in.Platform, in.PUid)
	if err != nil {
		if !errors.Is(err, model.ErrNotFound) {
			return nil, err // sql错误
		}
	}
	// 根据 updateTime 判断是否需要更新
	if dbPlatformUser != nil && time.Now().Add(time.Duration(l.svcCtx.Config.User.UpdateDuration)*time.Hour).Before(dbPlatformUser.UpdateTime) {
		var username *string
		if one, err := l.svcCtx.UserModel.FindOne(l.ctx, dbPlatformUser.UserId); err != nil {
			username = &one.Username
		}
		return &pb.PlatformUserResp{
			User: l.buildPBUser(username, dbPlatformUser),
		}, nil
	}

	var platformUser *platform_user.PlatformUser
	platformUser, err = l.platformUserRetriever.RetrieveUser(in.Platform, in.PUid)
	if err != nil {
		// 获取平台信息失败
		return nil, errors.Wrap(err, "获取平台信息失败")
	}

	var dbUser model.User
	if dbPlatformUser == nil {
		// create
		dbUser = model.User{
			Id:       uuid.NextId(),
			Username: "",
		}
		if _, err = l.svcCtx.UserModel.Insert(l.ctx, &dbUser); err != nil {
			return nil, errors.Wrap(err, "插入用户数据失败")
		}
		dbPlatformUser = &model.UserPlatform{
			Id:       uuid.NextId(),
			UserId:   dbUser.Id,
			Platform: in.Platform,
			POpenid:  "",
			PUid:     platformUser.PUid,
			PUname:   platformUser.PName,
			PInfo:    platformUser.PInfo,
			PAvatar:  platformUser.PAvatar,
		}
		if _, err = l.svcCtx.UserPlatformModel.Insert(l.ctx, dbPlatformUser); err != nil {
			return nil, errors.Wrap(err, "插入平台用户数据失败")
		}
	} else {
		// update
		dbPlatformUser.PUid = platformUser.PUid
		dbPlatformUser.PUname = platformUser.PName
		dbPlatformUser.PAvatar = platformUser.PAvatar
		dbPlatformUser.PInfo = platformUser.PInfo
		if err = l.svcCtx.UserPlatformModel.Update(l.ctx, dbPlatformUser); err != nil {
			return nil, errors.Wrap(err, "更新平台用户数据失败")
		}
	}
	return &pb.PlatformUserResp{
		User: l.buildPBUser(&dbUser.Username, dbPlatformUser),
	}, nil
}

func (l *RetrievePlatformUserLogic) buildPBUser(username *string, dbPlatformUser *model.UserPlatform) *pb.User {
	result := &pb.User{
		Id:       dbPlatformUser.UserId,
		Platform: dbPlatformUser.Platform,
		PUid:     dbPlatformUser.PUid,
		PUname:   dbPlatformUser.PUname,
		PAvatar:  dbPlatformUser.PAvatar,
		PInfo:    dbPlatformUser.PInfo,
	}
	if username != nil {
		result.Username = *username
	}
	return result
}
