package danmaku

import (
	"context"
	"git.noahlan.cn/northlan/ntools-go/kafka"
	"github.com/Shopify/sarama"
	"github.com/zeromicro/go-zero/core/logx"
	"google.golang.org/protobuf/proto"
	pbMq "live-service/app/pb/mq"
	"live-service/app/user_center/rpc/internal/server"
	"live-service/app/user_center/rpc/internal/svc"
	"live-service/app/user_center/rpc/pb"
	kfk "live-service/common/kafka"
	"strconv"
	"sync"
)

type msgHandlerFunc func(data []byte, msgKey string)

type MsgToDBHandler struct {
	context context.Context
	ctx     *svc.ServiceContext
	svr     *server.UserCenterServer

	msgHandle map[string]msgHandlerFunc

	cmdParser      *Parser
	commandManager *Manager

	ConsumerGroup *kafka.ConsumerGroup

	mutex sync.Mutex
	logx.Logger
}

func (h *MsgToDBHandler) Init(context context.Context, ctx *svc.ServiceContext, svr *server.UserCenterServer) {
	h.context = context
	h.ctx = ctx
	h.svr = svr
	h.Logger = logx.WithContext(h.context)

	cfg := ctx.Config.Danmaku.Kafka
	h.msgHandle = make(map[string]msgHandlerFunc)
	h.msgHandle["danmaku"] = h.handleDanmaku

	h.cmdParser = NewCMDParser(ctx.Config.Danmaku.Command.Keys)
	//
	h.commandManager = NewManager()
	h.commandManager.Register(h.handleJoinGame, "j", "J", "加入", "加入游戏")

	var err error
	h.ConsumerGroup, err = kafka.NewConsumerGroup(&kafka.ConsumerGroupConfig{
		KafkaVersion:   sarama.V3_1_0_0,
		OffsetsInitial: sarama.OffsetNewest,
		IsReturnErr:    false,
		UnMarshaler:    kfk.ProtobufMarshaler,
	}, cfg.Addr, []string{cfg.Topic}, cfg.ConsumerGroupId)

	if err != nil {
		h.Logger.Error(err)
	}
}

func (h *MsgToDBHandler) handleJoinGame(_ int64, _ string, danmaku *pbMq.MqDanmaku) {
	h.mutex.Lock()
	defer h.mutex.Unlock()
	_, err := h.svr.RetrievePlatformUser(h.context, &pb.PlatformUserReq{
		Platform: danmaku.Platform,
		PUid:     strconv.FormatInt(danmaku.Uid, 10),
	})
	if err != nil {
		return
	}
}

func (h *MsgToDBHandler) handleDanmaku(data []byte, msgKey string) {
	// msg proto
	var msgFromMq pbMq.MqDanmaku
	if err := proto.Unmarshal(data, &msgFromMq); err != nil {
		h.Logger.Error("unmarshal msg err", err)
		return
	}

	cmdStruct := h.cmdParser.Parse(msgFromMq.Content)
	// 只处理命令
	if cmdStruct.IsCMD {
		for _, cmd := range cmdStruct.Arr {
			h.commandManager.Handle(msgFromMq.LiveRoomId, cmd, &msgFromMq)
		}
	}
}

func (MsgToDBHandler) Setup(_ sarama.ConsumerGroupSession) error   { return nil }
func (MsgToDBHandler) Cleanup(_ sarama.ConsumerGroupSession) error { return nil }
func (h *MsgToDBHandler) ConsumeClaim(sess sarama.ConsumerGroupSession, claim sarama.ConsumerGroupClaim) error {
	for msg := range claim.Messages() {
		if hFunc, ok := h.msgHandle[msg.Topic]; ok {
			hFunc(msg.Value, string(msg.Key))
		}
		sess.MarkMessage(msg, "")
	}
	return nil
}
